#ifndef SFA_H_
#define SFA_H_

#include "DFT.h"
#include "TimeSeries.h"

#include <iostream>
#include <map>
#include <vector>

class SFA {
private:

	std::vector< std::vector<double> > lookuptable;
	std::vector< std::map<double, int> > multiHistogram;


	unsigned int histogramType = EQUI_DEPTH;
	unsigned int count = 0;
	unsigned int startOffset = 0;

	int symbols;

	void initHistogram();
	void createHistogram(const std::vector< std::shared_ptr<TimeSeries> > & samples, int from);

	void divideByBreakPoints(double* data, unsigned int dataSize, unsigned short* signal);
	void divideEquiDepthHistogram (int from);
	void divideEquiWidthHistogram (int from);

public:
	static const unsigned int EQUI_FREQUENCY = 0;
	static const unsigned int EQUI_DEPTH = 1;

    unsigned int* bestCoefficients;
    unsigned int coefficients;

	DFT* transformation;
	unsigned int maxCoefficient;

	SFA(unsigned int histogramType, unsigned int windowSize, unsigned int coefficients, unsigned int symbols, bool normMean);
	virtual ~SFA();

	void divideHistogram(const std::vector< std::shared_ptr<TimeSeries> > & samples, int from);
	void printHistogram();
	void lookup(TimeSeries & signature, unsigned short* word);
	void lookup(double* data, unsigned int dataSize, unsigned short* word);

    static void printData(unsigned int* data, int size);
    static void printData(double* data, int size);
	static void test();
};

#endif /* SFA_H_ */
