#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <vector>
#include <cmath>
#include <functional>
#include <iostream>
#include <sparsehash/dense_hash_map>
#include <set>
#include <chrono>

#include "SFA.h"
#include "MFT.h"
#include "TimeSeries.h"

struct WindowSizes {
	int accuracy;
	int windowSize;
	int symbols;
	int features;
	bool normed;

	std::shared_ptr<SFA> signature;
	std::vector< std::vector<int> > words;

    WindowSizes() :
        accuracy(0), windowSize(0), symbols(0), features(0),
        normed(false), signature(NULL), words() {
        
    }
    
    WindowSizes(int acc,
                int window,
                int s,
                int f,
                bool norm,
                std::shared_ptr<SFA> sig,
                std::vector< std::vector<int> > w)
        : accuracy(acc), windowSize(window), symbols(s), features(f),
          normed(norm), signature(sig), words(w) {
    }    
    
    void clear() {
        accuracy=0;
        words.clear();
	}
};

struct Predictions {
	std::vector<double> labels;
	int correct;

	Predictions(int cor, size_t labelSize) : labels(labelSize) {
		correct = cor;
	}

	Predictions(std::vector<double> l, int c) : labels(l), correct(c) {
	}
};

struct BagOfPattern {
    google::dense_hash_map<int,short> wordFrequencies;
    double label;
    
    BagOfPattern() : wordFrequencies(), label(0){
        wordFrequencies.set_empty_key(INT_MAX);
	}

    BagOfPattern(size_t size, double l) : wordFrequencies(size), label(l) {
        wordFrequencies.set_empty_key(INT_MAX);
	}
};

struct Statistic {
    google::dense_hash_map<int, float> tfIdf;

    Statistic(int size) : tfIdf(size) {
        tfIdf.set_empty_key(INT_MAX);
    }
    void clear() {
        tfIdf.clear();
    }
};

void loadDatset(char*);

int predictEuclideanDistance(
		const std::vector< std::shared_ptr<TimeSeries> > & testSamples,
		const std::vector< std::shared_ptr<TimeSeries> > & trainSamples);

double getEuclideanDistance(
		std::shared_ptr<TimeSeries> t1,
		std::shared_ptr<TimeSeries> t2,
		double minValue);

std::vector<WindowSizes> fit(
		int maxWindowLength,
		int minWindowLength,
		int& accuracy,
		int& windowSize,
		bool norm);

const Predictions predict(
		const std::vector< std::shared_ptr<TimeSeries> > & testSamples,
        const std::vector<int>& indices,
		const std::vector<BagOfPattern>& bagOfPatterns,
		const google::dense_hash_map< double, std::shared_ptr<Statistic> >& matrixTrain);

void createBagOfPatterns(
		std::vector<BagOfPattern>& bagOfPatterns,
        const std::vector< std::shared_ptr<TimeSeries> > & samples,
        const std::vector<std::vector <int> > & words,
		int features,
		int symbols);

void calcTfIdf(
        std::vector<BagOfPattern>& bagOfPatterns,
        google::dense_hash_map< double, std::shared_ptr<Statistic> >* matrix,
        //google::dense_hash_map< int, ushort >  & wordInClassFreq,
        const std::vector< std::shared_ptr<TimeSeries> > & samples,
        const std::vector<int>& indices,
        const std::set<double> & uniqueLabels,
        const std::vector<std::vector <int> > & words,
        int features,
        int symbols);
    
std::shared_ptr<SFA> createWords(
		uint windowSize,
		uint maxFeatures,
		uint maxSymbols,
		const std::vector< std::shared_ptr<TimeSeries> > & samples,
		std::vector<std::vector <int> > & words,
        std::shared_ptr<SFA> sig,
		bool normMean);

void normalizeStatistics(
        google::dense_hash_map< double, std::shared_ptr<Statistic> > & classStatistics);

const std::set<double> uniqueClassLabels(
        const std::vector< std::shared_ptr<TimeSeries> > & samples);

int predictEnsemble(
		const std::vector<WindowSizes>& results,
		int bestKnownCorrect,
		bool normMean,
		const std::vector< std::shared_ptr<TimeSeries> > & testSamples,
		const std::vector< std::shared_ptr<TimeSeries> > & trainSamples);

void initMatrix(
        google::dense_hash_map< double, std::shared_ptr<Statistic> >* matrix,
        const std::set<double> & uniqueLabels,
        const std::vector< std::vector<int> > & words);

std::vector< std::vector<int> > getStratifiedTrainTestSplitIndices(
        const std::vector< std::shared_ptr<TimeSeries> > & samples);

std::vector<int> getSplit(const std::vector< std::vector<int> >& setToSplit, uint exclude);

void outputResult(
		int correct, std::chrono::time_point<std::chrono::system_clock> & startTime, size_t testSize);

double formatError(int correct, size_t testSize);